@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2022 Meltytech, LLC
+ * Copyright (c) 2013-2024 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -77,7 +77,8 @@ public:
         IsBottomVideoRole,/// track only
         IsTopAudioRole,   /// track only
         IsBottomAudioRole,/// track only
-        AudioIndexRole    /// clip only
+        AudioIndexRole,   /// clip only
+        GroupRole,        /// clip only
     };
 
     explicit MultitrackModel(QObject *parent = 0);
@@ -100,7 +101,7 @@ public:
     QModelIndex makeIndex(int trackIndex, int clipIndex) const;
     QModelIndex parent(const QModelIndex &index) const;
     QHash<int, QByteArray> roleNames() const;
-    Q_INVOKABLE void audioLevelsReady(const QModelIndex &index);
+    Q_INVOKABLE void audioLevelsReady(const QPersistentModelIndex &index);
     bool createIfNeeded();
     void addBackgroundTrack();
     int addAudioTrack();
@@ -128,7 +129,7 @@ public:
 
 signals:
     void created();
-    void loaded();
+    void aboutToClose();
     void closed();
     void modified();
     void seeked(int position, bool seekPlayer = true);
@@ -141,6 +142,8 @@ signals:
     void appended(int trackIndex, int clipIndex);
     void inserted(int trackIndex, int clipIndex);
     void overWritten(int trackIndex, int clipIndex);
+    void removing(Mlt::Service *service);
+    void noMoreEmptyTracks(bool isAudio);
 
 public slots:
     void refreshTrackList();
@@ -160,7 +163,7 @@ public slots:
                       bool notify = true);
     int insertClip(int trackIndex, Mlt::Producer &clip, int position, bool rippleAllTracks,
                    bool seek = true, bool notify = true);
-    int appendClip(int trackIndex, Mlt::Producer &clip);
+    int appendClip(int trackIndex, Mlt::Producer &clip, bool seek = true, bool notify = true);
     void removeClip(int trackIndex, int clipIndex, bool rippleAllTracks);
     void liftClip(int trackIndex, int clipIndex);
     void splitClip(int trackIndex, int clipIndex, int position);
@@ -216,6 +219,7 @@ private:
     Mlt::Transition *getVideoBlendTransition(int trackIndex) const;
     void refreshVideoBlendTransitions();
     int bottomVideoTrackMltIndex() const;
+    bool hasEmptyTrack(TrackType trackType) const;
 
     friend class UndoHelper;
 
