@@ -43,8 +43,7 @@ QmlFilter::QmlFilter()
     , m_service(mlt_service(nullptr))
     , m_producer(mlt_producer(nullptr))
     , m_isNew(false)
-    , m_changeCommand(nullptr)
-    , m_changeCommandPushed(false)
+    , m_changeInProgress(0)
 {
     connect(this, SIGNAL(inChanged(int)), this, SIGNAL(durationChanged()));
     connect(this, SIGNAL(outChanged(int)), this, SIGNAL(durationChanged()));
@@ -57,10 +56,8 @@ QmlFilter::QmlFilter(Mlt::Service &mltService, const QmlMetadata *metadata, QObj
     , m_producer(mlt_producer(nullptr))
     , m_path(m_metadata->path().absolutePath().append('/'))
     , m_isNew(false)
-    , m_changeCommand(nullptr)
-    , m_changeCommandPushed(false)
+    , m_changeInProgress(false)
 {
-
     if (m_service.type() == mlt_service_filter_type) {
         // Every attached filter has a service property that points to the service to which it is attached.
         m_producer = Mlt::Producer(mlt_producer(m_service.is_valid() ? m_service.get_data("service") : 0));
@@ -72,10 +69,6 @@ QmlFilter::QmlFilter(Mlt::Service &mltService, const QmlMetadata *metadata, QObj
 
 QmlFilter::~QmlFilter()
 {
-    if (m_changeCommand && !m_changeCommandPushed) {
-        delete m_changeCommand;
-        m_changeCommand = nullptr;
-    }
 }
 
 QString QmlFilter::get(QString name, int position)
@@ -187,7 +180,7 @@ void QmlFilter::set(QString name, QString value, int position)
         if (qstrcmp(m_service.get(qUtf8Printable(name)), qUtf8Printable(value)))  {
             m_service.set_string(qUtf8Printable(name), qUtf8Printable(value)) ;
             emit changed(name);
-            updateChangeCommand(name);
+            updateUndoCommand(name);
         }
     } else {
         // Only set an animation keyframe if it does not already exist with the same value.
@@ -196,7 +189,7 @@ void QmlFilter::set(QString name, QString value, int position)
                 || value != m_service.anim_get(qUtf8Printable(name), position, duration())) {
             m_service.anim_set(qUtf8Printable(name), qUtf8Printable(value), position, duration());
             emit changed(name);
-            updateChangeCommand(name);
+            updateUndoCommand(name);
         }
     }
 }
@@ -210,7 +203,7 @@ void QmlFilter::set(QString name, const QColor &value, int position, mlt_keyfram
                 || value != QColor(mltColor.r, mltColor.g, mltColor.b, mltColor.a)) {
             m_service.set(qUtf8Printable(name), Util::mltColorFromQColor(value));
             emit changed(name);
-            updateChangeCommand(name);
+            updateUndoCommand(name);
         }
     } else {
         // Only set an animation keyframe if it does not already exist with the same value.
@@ -220,7 +213,7 @@ void QmlFilter::set(QString name, const QColor &value, int position, mlt_keyfram
                 || value != QColor(mltColor.r, mltColor.g, mltColor.b, mltColor.a)) {
             m_service.anim_set(qUtf8Printable(name), Util::mltColorFromQColor(value), position, duration());
             emit changed(name);
-            updateChangeCommand(name);
+            updateUndoCommand(name);
         }
     }
 }
@@ -239,7 +232,7 @@ void QmlFilter::set(QString name, double value, int position, mlt_keyframe_type
             } else if (name == "out") {
                 emit outChanged(delta);
             }
-            updateChangeCommand(name);
+            updateUndoCommand(name);
         }
     } else {
         // Only set an animation keyframe if it does not already exist with the same value.
@@ -249,7 +242,7 @@ void QmlFilter::set(QString name, double value, int position, mlt_keyframe_type
             mlt_keyframe_type type = getKeyframeType(animation, position, keyframeType);
             m_service.anim_set(qUtf8Printable(name), value, position, duration(), type);
             emit changed(name);
-            updateChangeCommand(name);
+            updateUndoCommand(name);
         }
     }
 }
@@ -268,7 +261,7 @@ void QmlFilter::set(QString name, int value, int position, mlt_keyframe_type key
             } else if (name == "out") {
                 emit outChanged(delta);
             }
-            updateChangeCommand(name);
+            updateUndoCommand(name);
         }
     } else {
         // Only set an animation keyframe if it does not already exist with the same value.
@@ -278,7 +271,7 @@ void QmlFilter::set(QString name, int value, int position, mlt_keyframe_type key
             mlt_keyframe_type type = getKeyframeType(animation, position, keyframeType);
             m_service.anim_set(qUtf8Printable(name), value, position, duration(), type);
             emit changed(name);
-            updateChangeCommand(name);
+            updateUndoCommand(name);
         }
     }
 }
@@ -298,7 +291,7 @@ void QmlFilter::set(QString name, double x, double y, double width, double heigh
                 || width != rect.w || height != rect.h || opacity != rect.o) {
             m_service.set(qUtf8Printable(name), x, y, width, height, opacity);
             emit changed(name);
-            updateChangeCommand(name);
+            updateUndoCommand(name);
         }
     } else {
         mlt_rect rect = m_service.anim_get_rect(qUtf8Printable(name), position, duration());
@@ -318,7 +311,7 @@ void QmlFilter::set(QString name, double x, double y, double width, double heigh
             mlt_keyframe_type type = getKeyframeType(animation, position, keyframeType);
             m_service.anim_set(qUtf8Printable(name), rect, position, duration(), type);
             emit changed(name);
-            updateChangeCommand(name);
+            updateUndoCommand(name);
         }
     }
 }
@@ -334,7 +327,7 @@ void QmlFilter::setGradient(QString name, const QStringList &gradient)
         }
     }
     emit changed(name.toUtf8().constData());
-    updateChangeCommand(name);
+    updateUndoCommand(name);
 }
 
 void QmlFilter::set(QString name, const QRectF &rect, int position, mlt_keyframe_type keyframeType)
@@ -527,29 +520,6 @@ int QmlFilter::framesFromTime(const QString &time)
     return 0;
 }
 
-QString QmlFilter::timeFromFrames(int frames, QmlFilter::TimeFormat format)
-{
-    if (MLT.producer()) {
-        mlt_time_format mltFormat = mlt_time_smpte_df;
-        switch ( format ) {
-        case TIME_FRAMES:
-            mltFormat = mlt_time_frames;
-            break;
-        case TIME_CLOCK:
-            mltFormat = mlt_time_clock;
-            break;
-        case TIME_TIMECODE_DF:
-            mltFormat = mlt_time_smpte_df;
-            break;
-        case TIME_TIMECODE_NDF:
-            mltFormat = mlt_time_smpte_ndf;
-            break;
-        }
-        return MLT.producer()->frames_to_time(frames, mltFormat);
-    }
-    return QString();
-}
-
 void QmlFilter::getHash()
 {
     if (m_service.is_valid())
@@ -631,7 +601,7 @@ void QmlFilter::setAnimateIn(int value)
                 }
             }
         }
-        updateChangeCommand(kShotcutAnimInProperty);
+        updateUndoCommand(kShotcutAnimInProperty);
         emit animateInChanged();
     }
 }
@@ -660,7 +630,7 @@ void QmlFilter::setAnimateOut(int value)
                 }
             }
         }
-        updateChangeCommand(kShotcutAnimOutProperty);
+        updateUndoCommand(kShotcutAnimOutProperty);
         emit animateOutChanged();
     }
 }
@@ -793,17 +763,121 @@ int QmlFilter::keyframeIndex(Mlt::Animation &animation, int position)
     return result;
 }
 
-void QmlFilter::updateChangeCommand(const QString &name)
+void QmlFilter::startUndoTracking()
 {
-    if (m_changeCommand) {
-        m_changeCommand->update(name);
-        if (!m_changeCommandPushed) {
-            MAIN.undoStack()->push(m_changeCommand);
-            auto index = MAIN.undoStack()->count() - 1;
-            m_changeCommand = (Filter::ChangeParameterCommand *) MAIN.undoStack()->command(index);
-            m_changeCommandPushed = true;
-        }
+    m_previousState = Mlt::Properties();
+    m_previousState.inherit(m_service);
+    if (!m_previousState.property_exists(kShotcutAnimInProperty)) {
+        m_previousState.set(kShotcutAnimInProperty, 0);
     }
+    if (!m_previousState.property_exists(kShotcutAnimOutProperty)) {
+        m_previousState.set(kShotcutAnimOutProperty, 0);
+    }
+}
+
+void QmlFilter::startUndoParameterCommand(const QString &desc)
+{
+    if (!m_previousState.count()) {
+//        LOG_DEBUG() << "Undo tracking has not started yet";
+        return;
+    }
+    m_changeInProgress++;
+    if (m_changeInProgress > 1) {
+//        LOG_DEBUG() << "Nested change command" << m_changeInProgress;
+        return;
+    }
+    auto command = new Filter::UndoParameterCommand(m_metadata->name(),
+                                                    MAIN.filterController(), MAIN.filterController()->currentIndex(), m_previousState, desc);
+    MAIN.undoStack()->push(command);
+}
+
+void QmlFilter::startUndoAddKeyframeCommand()
+{
+    if (!m_previousState.count()) {
+//        LOG_DEBUG() << "Undo tracking has not started yet";
+        return;
+    }
+    m_changeInProgress++;
+    if (m_changeInProgress > 1) {
+//        LOG_DEBUG() << "Nested change command" << m_changeInProgress;
+        return;
+    }
+    auto command = new Filter::UndoAddKeyframeCommand(m_metadata->name(),
+                                                      MAIN.filterController(), MAIN.filterController()->currentIndex(), m_previousState);
+    MAIN.undoStack()->push(command);
+}
+
+void QmlFilter::startUndoRemoveKeyframeCommand()
+{
+    if (!m_previousState.count()) {
+//        LOG_DEBUG() << "Undo tracking has not started yet";
+        return;
+    }
+    m_changeInProgress++;
+    if (m_changeInProgress > 1) {
+//        LOG_DEBUG() << "Nested change command" << m_changeInProgress;
+        return;
+    }
+    auto command = new Filter::UndoRemoveKeyframeCommand(
+        m_metadata->name(), MAIN.filterController(), MAIN.filterController()->currentIndex(),
+        m_previousState);
+    MAIN.undoStack()->push(command);
+}
+
+void QmlFilter::startUndoModifyKeyframeCommand(int paramIndex, int keyframeIndex)
+{
+    if (!m_previousState.count()) {
+//        LOG_DEBUG() << "Undo tracking has not started yet";
+        return;
+    }
+    m_changeInProgress++;
+    if (m_changeInProgress > 1) {
+        //        LOG_DEBUG() << "Nested change command" << m_changeInProgress;
+        return;
+    }
+    auto command = new Filter::UndoModifyKeyframeCommand(
+        m_metadata->name(), MAIN.filterController(), MAIN.filterController()->currentIndex(),
+        m_previousState, paramIndex, keyframeIndex);
+    MAIN.undoStack()->push(command);
+}
+
+void QmlFilter::updateUndoCommand(const QString &name)
+{
+    if (!m_previousState.count()) {
+//        LOG_DEBUG() << "Undo tracking has not started yet";
+        return;
+    }
+    if (m_changeInProgress) {
+        const QUndoCommand *lastCommand = MAIN.undoStack()->command(MAIN.undoStack()->count() - 1);
+        Filter::UndoParameterCommand *command = dynamic_cast<Filter::UndoParameterCommand *>
+                                                (const_cast<QUndoCommand *>(lastCommand));
+        if (command) {
+            // Update the change that is already in progress
+            command->update(name);
+        } else {
+            LOG_ERROR() << "Unable to find command in progress";
+            return;
+        }
+    } else {
+        // Nothing in progress... make a generic command for this update
+        auto command = new Filter::UndoParameterCommand(m_metadata->name(), MAIN.filterController(),
+                                                        MAIN.filterController()->currentIndex(), m_previousState);
+        MAIN.undoStack()->push(command);
+    }
+    m_previousState.pass_property(m_service, name.toUtf8().constData());
+}
+
+void QmlFilter::endUndoCommand()
+{
+    if (!m_previousState.count()) {
+//        LOG_DEBUG() << "Undo tracking has not started yet";
+        return;
+    }
+    if (!m_changeInProgress) {
+        LOG_ERROR() << "Change is not in progress";
+        return;
+    }
+    m_changeInProgress--;
 }
 
 mlt_keyframe_type QmlFilter::getKeyframeType(Mlt::Animation &animation, int position,
@@ -879,14 +953,6 @@ bool QmlFilter::allowAnimateOut() const
     return false;
 }
 
-void QmlFilter::startUndoTracking(FilterController *controller, int row)
-{
-    if (m_changeCommand) {
-        delete m_changeCommand;
-    }
-    m_changeCommand = new Filter::ChangeParameterCommand(m_metadata->name(), controller, row);
-}
-
 void QmlFilter::copyParameters()
 {
     auto name = "color";
