@@ -35,9 +35,6 @@ class AbstractJob;
 class EncodeJob;
 class QUndoCommand;
 class FilterController;
-namespace Filter {
-class ChangeParameterCommand;
-}
 
 class QmlFilter : public QObject
 {
@@ -53,14 +50,6 @@ class QmlFilter : public QObject
     Q_PROPERTY(bool blockSignals READ signalsBlocked WRITE blockSignals)
 
 public:
-    enum TimeFormat {
-        TIME_FRAMES,
-        TIME_CLOCK,
-        TIME_TIMECODE_DF,
-        TIME_TIMECODE_NDF,
-    };
-    Q_ENUM(TimeFormat)
-
     enum CurrentFilterIndex {
         NoCurrentFilter = -1,
         DeselectCurrentFilter = -2
@@ -116,8 +105,6 @@ public:
     Q_INVOKABLE void deletePreset(const QString &name);
     Q_INVOKABLE void analyze(bool isAudio = false, bool deferJob = true);
     Q_INVOKABLE static int framesFromTime(const QString &time);
-    Q_INVOKABLE static QString timeFromFrames(int frames,
-                                              QmlFilter::TimeFormat format = TIME_TIMECODE_DF);
     Q_INVOKABLE void getHash();
     Mlt::Producer &producer()
     {
@@ -149,13 +136,21 @@ public:
     bool allowAnimateIn() const;
     bool allowAnimateOut() const;
     Q_INVOKABLE void crop(const QRectF &rect);
-    void startUndoTracking(FilterController *controller, int row);
+
     Q_INVOKABLE void copyParameters();
     Q_INVOKABLE void pasteParameters(const QStringList &propertyNames);
 
+    // Functions for undo/redo
+    void startUndoTracking();
+    Q_INVOKABLE void startUndoParameterCommand(const QString &desc = QString());
+    void startUndoAddKeyframeCommand();
+    void startUndoRemoveKeyframeCommand();
+    void startUndoModifyKeyframeCommand(int paramIndex, int keyframeIndex);
+    void updateUndoCommand(const QString &name);
+    Q_INVOKABLE void endUndoCommand();
+
 public slots:
     void preset(const QString &name);
-    void updateChangeCommand(const QString &name);
 
 signals:
     void presetsChanged();
@@ -176,8 +171,8 @@ private:
     QString m_path;
     bool m_isNew;
     QStringList m_presets;
-    Filter::ChangeParameterCommand *m_changeCommand;
-    bool m_changeCommandPushed;
+    Mlt::Properties m_previousState;
+    int m_changeInProgress;
 
     QString objectNameOrService();
     int keyframeIndex(Mlt::Animation &animation, int position);
