@@ -1274,7 +1274,18 @@ int ShotcutSettings::backupPeriod() const
     return settings.value("backupPeriod", 24 * 60).toInt();
 }
 
-void ShotcutSettings::setBackupPeriod(int seconds)
+void ShotcutSettings::setBackupPeriod(int minutes)
 {
-    settings.setValue("backupPeriod", seconds);
+    settings.setValue("backupPeriod", minutes);
+}
+
+mlt_time_format ShotcutSettings::timeFormat() const
+{
+    return (mlt_time_format)settings.value("timeFormat", mlt_time_smpte_df).toInt();
+}
+
+void ShotcutSettings::setTimeFormat(int format)
+{
+    settings.setValue("timeFormat", format);
+    emit timeFormatChanged();
 }
