@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2021-2022 Meltytech, LLC
+ * Copyright (c) 2021-2023 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,6 @@
 
 #include "editmarkerwidget.h"
 
-#include "Logger.h"
 #include "mltcontroller.h"
 #include "qmltypes/qmlapplication.h"
 #include "widgets/timespinbox.h"
@@ -127,6 +126,9 @@ void EditMarkerWidget::on_colorButton_clicked()
 {
     QColor color = QColor(m_colorLabel->text());
     QColorDialog dialog(color);
+#if defined(Q_OS_UNIX) && !defined(Q_OS_MAC)
+    dialog.setOptions(QColorDialog::DontUseNativeDialog);
+#endif
     dialog.setModal(QmlApplication::dialogModality());
     if (dialog.exec() == QDialog::Accepted) {
         auto newColor = dialog.currentColor();
