@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2022 Meltytech, LLC
+ * Copyright (c) 2022-2023 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 #include "mltcontroller.h"
 #include "qmltypes/qmlapplication.h"
 #include "dialogs/longuitask.h"
+#include "videowidget.h"
 #include <Logger.h>
 #include <QProcess>
 #include <QFile>
@@ -88,11 +89,12 @@ void GlaxnimateProducerWidget::on_colorButton_clicked()
     if (m_producer) {
         color = QColor(QFileInfo(m_producer->get("background")).baseName());
     }
-    QColorDialog dialog(color);
-    dialog.setOption(QColorDialog::ShowAlphaChannel);
-    dialog.setModal(QmlApplication::dialogModality());
-    if (dialog.exec() == QDialog::Accepted) {
-        auto newColor = dialog.currentColor();
+    QColorDialog::ColorDialogOptions flags = QColorDialog::ShowAlphaChannel;
+#if defined(Q_OS_UNIX) && !defined(Q_OS_MAC)
+    flags = flags | QColorDialog::DontUseNativeDialog;
+#endif
+    auto newColor = QColorDialog::getColor(color, this, QString(), flags);
+    if (newColor.isValid()) {
         auto rgb = newColor;
         auto transparent = QColor(0, 0, 0, 0);
         rgb.setAlpha(color.alpha());
@@ -119,8 +121,8 @@ static void modifyJsonValue(QJsonValue &destValue, const QString &path, const QJ
     const int indexOfSquareBracketOpen = path.indexOf('[');
     const int indexOfSquareBracketClose = path.indexOf(']');
 
-    const int arrayIndex = path.midRef(indexOfSquareBracketOpen + 1,
-                                       indexOfSquareBracketClose - indexOfSquareBracketOpen - 1).toInt();
+    const int arrayIndex = path.mid(indexOfSquareBracketOpen + 1,
+                                    indexOfSquareBracketClose - indexOfSquareBracketOpen - 1).toInt();
 
     const QString squareBracketPropertyName = path.left(indexOfSquareBracketOpen);
     const QString squareBracketSubPath = indexOfSquareBracketClose > 0 ? (path.mid(
@@ -377,7 +379,7 @@ void GlaxnimateIpcServer::ParentResources::setProducer(const Mlt::Producer &prod
         // hide this clip's video track and upper ones
         int trackIndex = m_producer.get_int(kTrackIndexProperty);
         QString s = QString::fromLatin1(m_producer.get(kMultitrackItemProperty));
-        QVector<QStringRef> parts = s.splitRef(':');
+        auto parts = s.split(':');
         if (parts.length() == 2) {
             trackIndex = parts[1].toInt();
         }
@@ -462,15 +464,12 @@ void GlaxnimateIpcServer::onReadyRead()
         } else {
             m_isProtocolValid = true;
         }
-    } else {
+    }
+    if (m_isProtocolValid && parent) {
         qreal time = -1.0;
         for (int i = 0; i < 1000 && !m_stream->atEnd(); i++) {
             *m_stream >> time;
         }
-        if (!parent || !parent->m_glaxnimateProducer
-                || !parent->m_glaxnimateProducer->is_valid()
-                || time < 0.0)
-            return;
 
         // Only if the frame number is different
         int frameNum = parent->m_producer.get_int(kPlaylistStartProperty) + toMltFps(
@@ -478,6 +477,13 @@ void GlaxnimateIpcServer::onReadyRead()
         if (frameNum != parent->m_frameNum) {
             LOG_DEBUG() << "glaxnimate time =" << time << "=> Shotcut frameNum =" << frameNum;
 
+            if (!parent || !parent->m_glaxnimateProducer
+                    || !parent->m_glaxnimateProducer->is_valid()
+                    || time < 0.0) {
+                MLT.seek(frameNum);
+                return;
+            }
+
             // Get the image from MLT
             parent->m_glaxnimateProducer->seek(frameNum);
             std::unique_ptr<Mlt::Frame> frame(parent->m_glaxnimateProducer->get_frame());
@@ -527,6 +533,22 @@ void GlaxnimateIpcServer::onSocketError(QLocalSocket::LocalSocketError socketErr
     }
 }
 
+void GlaxnimateIpcServer::onFrameDisplayed(const SharedFrame &frame)
+{
+    auto image  = frame.get_image(mlt_image_rgb);
+    if (image) {
+        auto width = frame.get_image_width();
+        auto height = frame.get_image_height();
+        QImage temp(width, height, QImage::Format_RGB888);
+        for (int i = 0; i < height; i++) {
+            ::memcpy(temp.scanLine(i), &image[i * 3 * width], temp.bytesPerLine());
+        }
+        if (copyToShared(temp) && parent) {
+            parent->m_frameNum = frame.get_position();
+        }
+    }
+}
+
 GlaxnimateIpcServer &GlaxnimateIpcServer::instance()
 {
     static GlaxnimateIpcServer instance;
@@ -570,12 +592,18 @@ void GlaxnimateIpcServer::launch(const Mlt::Producer &producer, QString filename
 {
     parent.reset(new ParentResources);
 
-    LongUiTask longTask(QObject::tr("Edit With Glaxnimate"));
-    auto future = QtConcurrent::run([this, &producer, &hideCurrentTrack]() {
-        parent->setProducer(producer, hideCurrentTrack);
-        return true;
-    });
-    longTask.wait<bool>(tr("Preparing Glaxnimate preview...."), future);
+    if (Settings.playerGPU()) {
+        parent->m_producer = producer;
+        connect(qobject_cast<Mlt::VideoWidget *>(MLT.videoWidget()), &Mlt::VideoWidget::frameDisplayed,
+                this, &GlaxnimateIpcServer::onFrameDisplayed);
+    } else {
+        LongUiTask longTask(QObject::tr("Edit With Glaxnimate"));
+        auto future = QtConcurrent::run([this, &producer, &hideCurrentTrack]() {
+            parent->setProducer(producer, hideCurrentTrack);
+            return true;
+        });
+        longTask.wait<bool>(tr("Preparing Glaxnimate preview...."), future);
+    }
 
     if (filename.isEmpty()) {
         filename = QString::fromUtf8(parent->m_producer.get("resource"));
