@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2019 Meltytech, LLC
+ * Copyright (c) 2019-2022 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@ PlaylistListView::PlaylistListView(QWidget *parent)
 
 void PlaylistListView::dropEvent(QDropEvent *event)
 {
-    QModelIndex index = indexAt(event->pos());
+    QModelIndex index = indexAt(event->position().toPoint());
     if (event->dropAction() == Qt::MoveAction && index.row() == -1) {
         event->acceptProposedAction();
         emit movedToEnd();
